/*
  File Name     : ft602_pch_inp.v 
  Department    : IC Design, FTDI SGP
  Author        : Do Ngoc Duong
  History       : 19 December 2016 

  Description   : This module provides video input for one channel

*/
module ft602_pch_inp (  
  //
  input  rst_n,
  input  sys_clk,
  input  fhd_clk,
  input  hd_clk,
  input  sd_clk,
  //Registers 
  input [7:0]  mode_select,
  input [15:0] control,
  input [31:0] v_width,
  input [15:0] l_valid,
  input [15:0] l_sg_eg, 
  input [15:0] h_blank,
  input [15:0] d_valid,
  input [15:0] d_sg_eg,
  //
  input [15:0] high_mark, 
  input [15:0] low_mark,
  output alive,
  output frame_drop,  
  //
  // HDMI card interface  
  input  hdmi_pclk,
  input  hdmi_vsync,
  input  hdmi_hsync,
  input  hdmi_de,
  input  [11:0] hdmi_rd,
  input  [11:0] hdmi_gd,
  input  [11:0] hdmi_bd,
  //
  input  fifo_rclk,
  input  fiford,
  output fifonempt,
  output [35:0] fifo_rdata
);
//**************** 
localparam MEM_ADB = 12;  
localparam MEM_LEN = 4096; 
//****************
wire gen_ena;
wire gen_pclk;
wire gen_vsync;
wire gen_hsync;
wire gen_de;
wire [35:0] gen_data; 
wire seq_ena;
wire [31:0] seq_data;
// 
ft602_patt_gen i0_ft602_patt_gen (
  // 
  .rst_n        (rst_n),
  .sys_clk      (sys_clk),
  //
  .mode_select  (mode_select),
  .control      (control),
  .v_width      (v_width),
  .l_valid      (l_valid),
  .l_sg_eg      (l_sg_eg), 
  .h_blank      (h_blank), 
  .d_valid      (d_valid),
  .d_sg_eg      (d_sg_eg),  
  //
  .fhd_clk      (fhd_clk),
  .hd_clk       (hd_clk),
  .sd_clk       (sd_clk),
  //   
  .seq_ena      (seq_ena),
  .seq_data     (seq_data),  
  .alive        (alive),
  .gen_ena      (gen_ena), 
  .gen_pclk     (gen_pclk),
  .gen_vsync    (gen_vsync),
  .gen_hsync    (gen_hsync),
  .gen_de       (gen_de),
  .gen_data     (gen_data)
);
//
wire fifo_wclk;
wire wrmark;
wire fifowr;
wire [35:0] wdata; 
// 
ft602_hdmi_rx i1_ft602_hdmi_rx (
  //
  .rst_n        (rst_n),
  .gen_ena      (1'b1),//gen_ena),
  .frame_drop   (frame_drop), 
  //
  .hdmi_pclk    (hdmi_pclk),
  .hdmi_vsync   (hdmi_vsync),
  .hdmi_hsync   (hdmi_hsync),
  .hdmi_de      (hdmi_de),
  .hdmi_rd      (hdmi_rd),
  .hdmi_gd      (hdmi_gd),
  .hdmi_bd      (hdmi_bd),
  //
  .seq_ena      (seq_ena),
  .seq_data     (seq_data),  
  //  
  .gen_pclk     (gen_pclk),
  .gen_vsync    (gen_vsync),
  .gen_hsync    (gen_hsync),
  .gen_de       (gen_de),
  .gen_data     (gen_data), 
  //
  .out_clk      (fifo_wclk),
  .wrmark       (wrmark),
  .fifowr       (fifowr),
  .wdata        (wdata)
);
//
//Dual clock FIFO 
wire               mem_wr;
wire [MEM_ADB-1:0] mem_wa;
wire [MEM_ADB-1:0] mem_ra;
//
ft602_dclk_fifo #(.ADB(MEM_ADB),.LEN(MEM_LEN))  i2_ft602_dclk_fifo (

  .wrclk       (fifo_wclk),
  .wrrst_n     (rst_n), 
  .fifowr      (fifowr),
  .wrfifoful   (),
  .wrfifolen   (),
  .wrmarklen   (high_mark[MEM_ADB:0]),
  .wrmark      (wrmark),

  .rdclk       (fifo_rclk),
  .rdrst_n     (rst_n),
  .fiford      (fiford),
  .rdfifonempt (),
  .rdfifolen   (),
  .rdmarklen   (low_mark[MEM_ADB:0]),
  .rdmark      (fifonempt),

  .write       (mem_wr),
  .wraddr      (mem_wa),
  .read        (),
  .rdaddr      (mem_ra)
);
//Buffer memory 
`ifdef ALTERA_FPGA 
dpsram_4kx36 i3_dpsram_4kx36 (
  .data       (wdata),
  .rdaddress  (mem_ra),
  .rdclock    (fifo_rclk),
  .wraddress  (mem_wa),
  .wrclock    (fifo_wclk),
  .wren       (mem_wr),
  .q          (fifo_rdata)
);
`else

dpsram_4kx36(
  .clka       (fifo_wclk),
  .wea        (mem_wr),
  .addra      (mem_wa),
  .dina       (wdata),
  .clkb       (fifo_rclk),
  .addrb      (mem_ra),
  .doutb      (fifo_rdata)
);

`endif

//   
endmodule 
